#ifndef GAMEMANAGER_INCLUDED
 #define GAMEMANAGER_INCLUDED

#include "vector3d.h"
#include "resourcemanager.h"
#include <vector>

class GameManager;

class GameObject {
	protected:
		GameManager *manager;
	public:
		bool dead;
		bool collidable; 
		Vect3d location;
		Vect3d velocity;
		float radius;
		
		void setManager(GameManager *m) {
			manager = m;
		}
		
		virtual void render() { }
		virtual void renderBackground() { }
		virtual void renderForeground() { }
		virtual void tick() { }
		virtual void collided(GameObject * other) { }
		
		virtual ~GameObject() { }
		
		GameObject() : dead(false), collidable(false) { }
};

struct Goal {
	Vect3d location;
	Component radius;
	bool satisfied;
	Goal() : radius(1.2), satisfied(false) { }
};

// Particle - created with life=maxlife.
// Will fade away to greater alpha as it proceeds.
struct Particle {
	Vect3d location;
	Vect3d velocity;
	Component radius;
	int maxlife;
	int life;
	float r,g,b;
	Particle(Vect3d where) : radius(0.1), life(100), r(1.0), g(1.0), b(1.0) {
		location = where;
		maxlife = life;
	}
};

class GameManager {
	private:
		typedef std::vector <GameObject *> GameObjectList;
		typedef GameObjectList::iterator GameObjectIterator;
		GameObjectList things;
		GameObjectList deadThings;
		GameObjectList newThings;
		
		typedef std::vector <Goal> GoalList;
		typedef std::vector <Particle> ParticleList;
		
		GoalList goals;
		ParticleList particles;
		
		void cleanList( GameObjectList &list);
		void handleCollision(GameObject *obj1, GameObject * obj2);
		void collideTest();
		void tickPointer();
		void tickParticles();
		void renderParticles();
		void renderPointer();
		void renderGoals();
		void resetGoals();
	public:
		GameObject *trackObject;
		Vect3d viewCentre;
		Vect3d viewOffset;
		
		int numStars;
		int goalsComplete;
		
		float pointerx; // world coords
		float pointery; // world coords
		
		GameManager();
		~GameManager() {
			cleanList(things);
			cleanList(newThings);
			cleanList(deadThings);
		}
		void render();
		void tick();
		
		void add(GameObject *thing) {
			thing->setManager(this);
			newThings.push_back(thing);
		}
		
		void setTrack(GameObject *what) {
			trackObject = what;
		}
		
		/* Used between levels to get rid of the previous
	 	 * Level's stuff.
		 */
		void cleanUp() 
		{
			trackObject = 0;
			cleanList(things);
			cleanList(newThings);
			cleanList(deadThings);
			particles.clear();
			goals.clear();
		}
		
		void triggerSound(ResourceManager::SampleType sample, const Vect3d &where);
		
		GameObject * findNearestCollidable(Vect3d where) {
			return findNearestCollidableExcept(where, 0);
		}
		
		GameObject * findNearestCollidableExcept(Vect3d where, GameObject * what);
		
		void registerStarPos(GameObject *star);
		
		int countGoals() { return goals.size(); }
		
		void addParticle(Particle &p)
		{
			p.maxlife = p.life;
			particles.push_back(p);
		}
		void createExplosion(GameObject *obj);
		void showRadar(Vect3d where, float r, float g, float b);
		
		void addGoal(Vect3d where)
		{
			Goal g;
			g.location = where;
			goals.push_back(g);
		}
};

#endif // GAMEMANAGER_INCLUDED

