#include "starfield.h"
#include "utils.h"

#include "constants.h"

#include <alleggl.h>
#include <GL/glu.h>
#include <stdexcept>
#include <cmath>

static const float StarFieldSize = ViewWidth * 1.5;
static const float MinStarDist = -2.0;
static const float MaxStarDist = -8.0;

void StarField::initTextures()
{
	// Render a 512 x 512 area
	// clear bg.
	glClearColor(0,0,0, 0);
	glClear(GL_DEPTH_BUFFER_BIT| GL_COLOR_BUFFER_BIT);
	// Render background.
	SetOrthoProjection();
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 
		resourceManager.getTexture(ResourceManager::StarTexture));
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
	
	// Draw a load of random stars of varying sizes.
	// For each star, draw it 9 times, wrapping around.
	const int NumBackStars = 500;
	for (int i=0; i < NumBackStars; i++) {
		int x = rand() % 512;
		int y = rand() % 512;
		int radius = 1 + (rand() % ((rand() % 4) + 1));
		// Make a few really big stars.
		if (i > 400) {
			radius *= 2;
		}
		
		for (int lx = -1; lx <=1 ; lx ++) {
			for (int ly = -1; ly <=1; ly ++) {
				float leftx = (float) (x - radius) + lx * 512;
				float rightx = (float) (x + radius) + lx * 512;
				float bottomy = (float) (y - radius) + ly * 512;
				float topy = (float) (y + radius) + ly * 512;
				// Draw a quad.
				glColor3f(1,1,1);
				glBegin(GL_QUADS);
				glTexCoord2f(0,0);
				glVertex2f(leftx,bottomy);
				glTexCoord2f(1,0);
				glVertex2f(rightx,bottomy);
				glTexCoord2f(1,1);
				glVertex2f(rightx,topy);
				glTexCoord2f(0,1);
				glVertex2f(leftx,topy);
				glEnd();
			}
		}
	}
	glGenTextures(1, & texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

	glReadBuffer(GL_BACK); 
	glCopyTexImage2D(
		GL_TEXTURE_2D,
		0, // mipmap level
		GL_RGB,
		0,0, // bottom left
		512, 512, // size
		0); // border.
	CheckGlError();
	
	glDisable(GL_BLEND);
	// Draw a quick quad to test it.
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex2f(500,500);
	glTexCoord2f(1,0);
	glVertex2f(600,500);
	glTexCoord2f(1,1);
	glVertex2f(600,600);
	glTexCoord2f(0,1);
	glVertex2f(500,600);
	glEnd();

	/*
	allegro_gl_flip();
	rest(2000); // hang around a while.
	*/
}

void StarField::initStars() 
{
	starpos.resize(NumStars);
	for (int i=0; i<NumStars; i++) {
		Vect3d &pos = starpos[i];
		
		pos.x = FloatRand() * StarFieldSize;
		pos.y = FloatRand() * StarFieldSize;
		pos.z = MinStarDist + (FloatRand() * (MaxStarDist - MinStarDist));
	}
}

void StarField::renderBackground()
{
	const float BlockSize = 10.0;
	float blockleft = (manager->viewCentre.x - (ViewWidth / 2)) / BlockSize;
	blockleft = std::floor(blockleft) * BlockSize;
	
	float blockbottom = (manager->viewCentre.y - (ViewHeight / 2)) / BlockSize;
	blockbottom = std::floor(blockbottom) * BlockSize;
	
	float blockright = blockleft + BlockSize*3;
	float blocktop = blockbottom + BlockSize*3;
	blockleft = blockleft - BlockSize;
	blockbottom = blockbottom - BlockSize;
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
	float starz = -10.0;
	// Draw a quad.
	glColor3f(0.7, 0.7, 0.7); // Not full bright.
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(blockleft,blockbottom, starz);
	glTexCoord2f(4,0);
	glVertex3f(blockright,blockbottom, starz);
	glTexCoord2f(4,4);
	glVertex3f(blockright,blocktop, starz);
	glTexCoord2f(0,4);
	glVertex3f(blockleft,blocktop, starz);
	glEnd();
	renderStars();
}

void StarField::renderStars()
{
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 
		resourceManager.getTexture(ResourceManager::StarTexture));
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
	
	glBegin(GL_QUADS);
	
	float blockleft = (manager->viewCentre.x - (StarFieldSize / 2)) / StarFieldSize;
	blockleft = std::floor(blockleft) * StarFieldSize;
	
	float blockbottom = (manager->viewCentre.y - (StarFieldSize / 2)) / StarFieldSize;
	blockbottom = std::floor(blockbottom) * StarFieldSize;
	
	float minx = (manager->viewCentre.x - (StarFieldSize / 2));
	float miny = (manager->viewCentre.y - (StarFieldSize / 2));
	
	for (int i=0; i < NumStars; i++) {
		Vect3d pos = starpos[i]; // Copy!
		pos.x += blockleft;
		pos.y += blockbottom;
		// If the star is off the bottom or left, move it over to the right.
		if (pos.x < minx) {
			pos.x += StarFieldSize;
		}
		if (pos.y < miny) {
			pos.y += StarFieldSize;
		}
		
		float radius = (((float) i / NumStars) * 0.5) + 0.2;
		float leftx = pos.x - radius;
		float rightx = pos.x + radius;
		float bottomy = pos.y - radius;
		float topy = pos.y + radius;
		
		float brightness = 0.9; 
		glColor3f(brightness, brightness, brightness);
		glTexCoord2f(0,0);
		glVertex3f(leftx,bottomy, pos.z);
		glTexCoord2f(1,0);
		glVertex3f(rightx,bottomy, pos.z);
		glTexCoord2f(1,1);
		glVertex3f(rightx,topy, pos.z);
		glTexCoord2f(0,1);
		glVertex3f(leftx,topy, pos.z);
	}
	glEnd();
}

void StarField::tick() 
{ 

}

StarField::~StarField()
{
	if (texture) {
		glDeleteTextures(1, & texture);
	}
}

